@@ -6,7 +6,7 @@ Mojo::IOLoop::ReadWriteFork - Fork a process and read/write from it
 
 =head1 VERSION
 
-0.11
+0.12
 
 =head1 DESCRIPTION
 
@@ -61,9 +61,15 @@ use constant DEBUG             => $ENV{MOJO_READWRITE_FORK_DEBUG} || $ENV{MOJO_R
 use constant WAIT_PID_INTERVAL => $ENV{WAIT_PID_INTERVAL}         || 0.01;
 use constant SIGCHLD => 'DEFAULT';    # no idea why I need to set SIGCHLD, but waitpid() misbehave if not
 
-sub ESC { Mojo::Util::url_escape($_[0], '^A-Za-z0-9\s\-._~'); }
+my %ESC = ("\0" => '\0', "\a" => '\a', "\b" => '\b', "\f" => '\f', "\n" => '\n', "\r" => '\r', "\t" => '\t');
 
-our $VERSION = '0.11';
+sub ESC {
+  local $_ = shift;
+  s/([\x00-\x1f\x7f\x80-\x9f])/$ESC{$1} || sprintf "\\x%02x", ord $1/ge;
+  $_;
+}
+
+our $VERSION = '0.12';
 
 our @SAFE_SIG = grep {
   not /^(
@@ -234,8 +240,10 @@ sub _start {
 
         # 5 = Input/output error
         if ($self->{errno} == 5) {
-          warn "[$pid] Ignoring child after $self->{errno}\n" if DEBUG;
-          $reactor->watch(delete $self->{stdout_read}, 0, 0);
+          if (my $handle = delete $self->{stdout_read}) {
+            warn "[$pid] Ignoring child after $self->{errno}\n" if DEBUG;
+            $reactor->watch($handle, 0, 0);
+          }
         }
         elsif ($self->{errno}) {
           warn "[$pid] Child $self->{errno}\n" if DEBUG;
@@ -378,7 +386,7 @@ sub _read {
 
   return unless defined $read;
   return unless $read;
-  warn "[$self->{pid}] Got buffer (@{[ESC($buffer)]})\n" if DEBUG;
+  warn "[$self->{pid}] >>> @{[ESC($buffer)]}\n" if DEBUG;
   $self->emit(read => $buffer);
 }
 
@@ -390,7 +398,7 @@ sub _write {
   my $written     = $stdin_write->syswrite($self->{stdin_buffer});
   return $self->_error unless defined $written;
   my $chunk = substr $self->{stdin_buffer}, 0, $written, '';
-  warn "[${ \$self->pid }] Wrote buffer (@{[ESC($chunk)]})\n" if DEBUG;
+  warn "[${ \$self->pid }] <<< @{[ESC($chunk)]}\n" if DEBUG;
 
   if (length $self->{stdin_buffer}) {
 
